<?php

namespace App\Http\Controllers;

use App\Models\JobModel;
use App\Models\JobPiece;
use App\Models\Stock;
use App\Models\StockUsed;
use App\Models\UserPiece;
use App\Models\Waste;
use Exception;
use Illuminate\Http\Request;

class JobController extends Controller
{
    public function jobs()
    {
        $jobs = JobModel::with('wastes', 'stock_used.stock') // Ensure relationships are correctly loaded
            ->orderBy('created_at', 'desc')
            ->get();
        // return $jobs;
        return view('admin.job.index', ['jobs' => $jobs]);
    }

    public function job_details($id){
        $job = JobModel::with('wastes') // Ensure relationships are correctly loaded
            ->orderBy('created_at', 'desc')
            ->where('id',$id)
            ->get();
            // return $job;
        return view('admin.job.show', ['jobs' => $job]);
    }

    public function add_job(Request $request)
    {
        try {
            // Step 1: Parse and initialize variables
            $jobLengths = $request->length; // Array of job lengths
            $stockPieces = Stock::orderBy('length', 'asc')->get(); // Fetch stock sorted ascending to prioritize smallest pieces
            $wastePool = []; // Track all waste pieces
            $jobInfo = []; // To store detailed information
    
            $job = new JobModel();
            $job->job_no = $request->job_no;
            $job->address = $request->address;
            $job->site = $request->site;
            $job->length = json_encode($jobLengths); // Save lengths as JSON array
            $job->save();
    
            foreach ($jobLengths as $length) {
                $requiredLength = (int) $length;
                $info = [
                    'length' => $requiredLength,
                    'source' => '',
                    'used_piece' => '',
                    'qty' => 0, // Quantity of the used piece
                    'remaining_waste' => ''
                ];
    
                // Step 2: Check waste pool for usable waste
                $fulfilled = false;
                foreach ($wastePool as $key => &$waste) {
                    if ($waste >= $requiredLength) {
                        $info['source'] = 'waste';
                        $info['used_piece'] = $waste;
                        $info['qty'] = 1; // Only one piece from the waste pool
    
                        $waste -= $requiredLength; // Deduct from waste
                        if ($waste > 0) {
                            $wastePool[$key] = $waste; // Update waste pool
                        } else {
                            unset($wastePool[$key]); // Remove empty waste
                        }
    
                        // Save the used piece from waste
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $requiredLength; // Length used
                        $usedPiece->type = 'waste'; // Type is waste
                        $usedPiece->qty = 1; // Qty used from waste
                        $usedPiece->save();
    
                        $fulfilled = true;
                        break;
                    }
                }
    
                if ($fulfilled) {
                    $info['remaining_waste'] = implode(", ", $wastePool);
                    $jobInfo[] = $info;
                    continue; // If fulfilled by waste, move to the next job length
                }
    
                // Step 3: Use stock if waste cannot fulfill
                foreach ($stockPieces as $key => $stock) {
                    if ($stock->length >= $requiredLength) {
                        $remainingWaste = $stock->length - $requiredLength; // Calculate remaining waste
    
                        $info['source'] = 'stock';
                        $info['used_piece'] = $stock->length;
                        $info['qty'] = 1; // Only one piece from the stock
    
                        // Save the used piece from stock
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $stock->length; // Full stock length used initially
                        $usedPiece->type = 'stock'; // Type is stock
                        $usedPiece->qty = 1; // Qty used from stock
                        $usedPiece->save();
    
                        if ($remainingWaste > 0) {
                            $wastePool[] = $remainingWaste; // Add remaining waste to pool
    
                            // Save remaining waste as a new UserPiece
                            $wastePiece = new UserPiece();
                            $wastePiece->job_model_id = $job->id;
                            $wastePiece->length = $remainingWaste;
                            $wastePiece->type = 'waste';
                            $wastePiece->qty = 1; // Mark the waste as 1 piece
                            $wastePiece->save();
                        }
    
                        unset($stockPieces[$key]); // Remove stock after usage
                        $info['remaining_waste'] = implode(", ", $wastePool);
                        $fulfilled = true;
                        break;
                    }
                }
    
                if (!$fulfilled && count($stockPieces) > 0) {
                    // Use the smallest piece available
                    $smallestStock = $stockPieces->first(); // Fetch the smallest stock
                    if ($smallestStock) {
                        $remainingWaste = $smallestStock->length - $requiredLength;
    
                        $info['source'] = 'stock';
                        $info['used_piece'] = $smallestStock->length;
                        $info['qty'] = 1; // Only one piece from the smallest stock
    
                        // Save the used piece from smallest stock
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $smallestStock->length;
                        $usedPiece->type = 'stock';
                        $usedPiece->qty = 1;
                        $usedPiece->save();
    
                        if ($remainingWaste > 0) {
                            $wastePool[] = $remainingWaste; // Add remaining waste to pool
    
                            // Save remaining waste as a new UserPiece
                            $wastePiece = new UserPiece();
                            $wastePiece->job_model_id = $job->id;
                            $wastePiece->length = $remainingWaste;
                            $wastePiece->type = 'waste';
                            $wastePiece->qty = 1;
                            $wastePiece->save();
                        }
    
                        $stockPieces = $stockPieces->filter(function ($stock) use ($smallestStock) {
                            return $stock->id !== $smallestStock->id;
                        }); // Remove smallest stock after usage
                    }
                }
    
                $info['remaining_waste'] = implode(", ", $wastePool);
                $jobInfo[] = $info;
            }
    
            // Finalize remaining waste
            $finalWaste = array_filter($wastePool, function ($waste) {
                return $waste > 0;
            });
    
            // Save all remaining waste
            foreach ($finalWaste as $wasteLength) {
                $waste = new Waste();
                $waste->length = $wasteLength;
                $waste->job_model_id = $job->id;
                $waste->save();
            }
    
            // Save job information
            $job->information = json_encode($jobInfo);
            $job->save();
    
            return redirect()->back()->with('success', 'Job added successfully');
    
        } catch (Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
    public function edit_job(Request $request)
    {
        try {
            // Step 1: Parse and initialize variables
            $jobLengths = $request->length; // Array of job lengths
            $stockPieces = Stock::orderBy('length', 'asc')->get(); // Fetch stock sorted ascending to prioritize smallest pieces
            $wastePool = []; // Track all waste pieces
            $jobInfo = []; // To store detailed information
    
            $job =  JobModel::find($request->job_id);
            $job->job_no = $request->job_no;
            $job->address = $request->address;
            $job->site = $request->site;
            $job->length = json_encode($jobLengths); // Save lengths as JSON array
            $job->save();

            $delete_waste = Waste::where('job_model_id',$request->job_id)->get();
            if(count($delete_waste) > 0){
                foreach($delete_waste as $dd){
                    $dd->delete();
                }
            }
    
            foreach ($jobLengths as $length) {
                $requiredLength = (int) $length;
                $info = [
                    'length' => $requiredLength,
                    'source' => '',
                    'used_piece' => '',
                    'qty' => 0, // Quantity of the used piece
                    'remaining_waste' => ''
                ];
    
                // Step 2: Check waste pool for usable waste
                $fulfilled = false;
                foreach ($wastePool as $key => &$waste) {
                    if ($waste >= $requiredLength) {
                        $info['source'] = 'waste';
                        $info['used_piece'] = $waste;
                        $info['qty'] = 1; // Only one piece from the waste pool
    
                        $waste -= $requiredLength; // Deduct from waste
                        if ($waste > 0) {
                            $wastePool[$key] = $waste; // Update waste pool
                        } else {
                            unset($wastePool[$key]); // Remove empty waste
                        }
    
                        // Save the used piece from waste
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $requiredLength; // Length used
                        $usedPiece->type = 'waste'; // Type is waste
                        $usedPiece->qty = 1; // Qty used from waste
                        $usedPiece->save();
    
                        $fulfilled = true;
                        break;
                    }
                }
    
                if ($fulfilled) {
                    $info['remaining_waste'] = implode(", ", $wastePool);
                    $jobInfo[] = $info;
                    continue; // If fulfilled by waste, move to the next job length
                }
    
                // Step 3: Use stock if waste cannot fulfill
                foreach ($stockPieces as $key => $stock) {
                    if ($stock->length >= $requiredLength) {
                        $remainingWaste = $stock->length - $requiredLength; // Calculate remaining waste
    
                        $info['source'] = 'stock';
                        $info['used_piece'] = $stock->length;
                        $info['qty'] = 1; // Only one piece from the stock
    
                        // Save the used piece from stock
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $stock->length; // Full stock length used initially
                        $usedPiece->type = 'stock'; // Type is stock
                        $usedPiece->qty = 1; // Qty used from stock
                        $usedPiece->save();
    
                        if ($remainingWaste > 0) {
                            $wastePool[] = $remainingWaste; // Add remaining waste to pool
    
                            // Save remaining waste as a new UserPiece
                            $wastePiece = new UserPiece();
                            $wastePiece->job_model_id = $job->id;
                            $wastePiece->length = $remainingWaste;
                            $wastePiece->type = 'waste';
                            $wastePiece->qty = 1; // Mark the waste as 1 piece
                            $wastePiece->save();
                        }
    
                        unset($stockPieces[$key]); // Remove stock after usage
                        $info['remaining_waste'] = implode(", ", $wastePool);
                        $fulfilled = true;
                        break;
                    }
                }
    
                if (!$fulfilled && count($stockPieces) > 0) {
                    // Use the smallest piece available
                    $smallestStock = $stockPieces->first(); // Fetch the smallest stock
                    if ($smallestStock) {
                        $remainingWaste = $smallestStock->length - $requiredLength;
    
                        $info['source'] = 'stock';
                        $info['used_piece'] = $smallestStock->length;
                        $info['qty'] = 1; // Only one piece from the smallest stock
    
                        // Save the used piece from smallest stock
                        $usedPiece = new UserPiece();
                        $usedPiece->job_model_id = $job->id;
                        $usedPiece->length = $smallestStock->length;
                        $usedPiece->type = 'stock';
                        $usedPiece->qty = 1;
                        $usedPiece->save();
    
                        if ($remainingWaste > 0) {
                            $wastePool[] = $remainingWaste; // Add remaining waste to pool
    
                            // Save remaining waste as a new UserPiece
                            $wastePiece = new UserPiece();
                            $wastePiece->job_model_id = $job->id;
                            $wastePiece->length = $remainingWaste;
                            $wastePiece->type = 'waste';
                            $wastePiece->qty = 1;
                            $wastePiece->save();
                        }
    
                        $stockPieces = $stockPieces->filter(function ($stock) use ($smallestStock) {
                            return $stock->id !== $smallestStock->id;
                        }); // Remove smallest stock after usage
                    }
                }
    
                $info['remaining_waste'] = implode(", ", $wastePool);
                $jobInfo[] = $info;
            }
    
            // Finalize remaining waste
            $finalWaste = array_filter($wastePool, function ($waste) {
                return $waste > 0;
            });
    
            // Save all remaining waste
            foreach ($finalWaste as $wasteLength) {
                $waste = new Waste();
                $waste->length = $wasteLength;
                $waste->job_model_id = $job->id;
                $waste->save();
            }
    
            // Save job information
            $job->information = json_encode($jobInfo);
            $job->save();
    
            return redirect()->back()->with('success', 'Job updated successfully');
    
        } catch (Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
    
   
    

    
    
    // public function add_job(Request $request)
    // {
    //     try {
    //         // Step 1: Parse and initialize variables
    //         $jobLengths = $request->length; // Array of job lengths
    //         $stockPieces = Stock::orderBy('length', 'desc')->get(); // Fetch stock sorted descending
    //         $wastePool = []; // Track all waste pieces
    //         $usedStocks = []; // Track all used stock IDs
    //         $wasteData = []; // To collect final waste information

    //                     $job = new JobModel();
    //         $job->job_no = $request->job_no;
    //         $job->address = $request->address;
    //         $job->site = $request->site;
    //         $job->length = json_encode($jobLengths); // Save lengths as JSON array
    //         $job->save();

    //         foreach ($jobLengths as $length) {
    //             $requiredLength = (int) $length;

    //             // Step 2: Check waste pool for usable waste
    //             $fulfilled = false;
    //             foreach ($wastePool as $key => &$waste) {
    //                 if ($waste >= $requiredLength) {
    //                     $waste -= $requiredLength; // Deduct from waste
    //                     if ($waste > 0) {
    //                         $wastePool[$key] = $waste; // Update waste pool
    //                     } else {
    //                         unset($wastePool[$key]); // Remove empty waste
    //                     }

    //                     // Save the used piece from waste
    //                     $usedPiece = new UserPiece();
    //                     $usedPiece->job_model_id = $job->id;
    //                     $usedPiece->length = $requiredLength; // Length used
    //                     $usedPiece->type = 'waste'; // Type is waste
    //                     $usedPiece->qty = 1;
    //                     $usedPiece->save();

    //                     $fulfilled = true;
    //                     break;
    //                 }
    //             }

    //             if ($fulfilled) {
    //                 continue; // If fulfilled by waste, move to the next job length
    //             }

    //             // Step 3: Use or reuse stock if waste cannot fulfill
    //             foreach ($stockPieces as $key => $stock) {
    //                 if ($stock->length >= $requiredLength) {
    //                     $remainingWaste = $stock->length - $requiredLength; // Calculate remaining waste
    //                     $wastePool[] = $remainingWaste; // Add remaining waste to pool
    //                     $usedStocks[] = $stock->id; // Track stock used

    //                     // Save the used piece from stock
    //                     $usedPiece = new UserPiece();
    //                     $usedPiece->job_model_id = $job->id;
    //                     $usedPiece->length = $requiredLength; // Length used
    //                     $usedPiece->type = 'stock'; // Type is stock
    //                     // $usedPiece->stock_id = $stock->id;
    //                     $usedPiece->qty = 1;
    //                     $usedPiece->save();

    //                     unset($stockPieces[$key]); // Remove stock after usage
    //                     $fulfilled = true;
    //                     break;
    //                 }
    //             }

    //             if (!$fulfilled) {
    //                 return response()->json(['error' => "Not enough stock to fulfill the job."], 400);
    //             }
    //         }
    //         $usedStocks = array_count_values($usedStocks);
    //         foreach ($usedStocks as $key => $value) {
    //             $sUsed = new StockUsed();
    //             $sUsed->job_model_id = $job->id;
    //             $sUsed->cost = $value;
    //             $sUsed->stock_id = $key;
    //             $sUsed->save();
               
    //         }
            
    //         // Step 4: Finalize waste pool
    //         $finalWaste = array_values(array_filter($wastePool, function ($waste) {
    //             return $waste > 0;
    //         }));
    //         $finalWaste = array_sum($finalWaste);
    //         $waste = new Waste();
    //         $waste->length = $finalWaste;
    //         $waste->job_model_id = $job->id;
    //         $waste->save();

    //         return redirect()->back()->with('success','Job added successfully');

           
    //     } catch (Exception $e) {
    //         return response()->json(['error' => $e->getMessage()], 500);
    //     }
    // }


    public function delete_job($id)
    {
        try {
            // Find the job
            $job = JobModel::find($id);

            if ($job) {
                // Delete all UserPiece records where job_model_id matches
                UserPiece::where('job_model_id', $id)->delete();

                // Delete the job
                $job->delete();

                return redirect()->back()->with('success', 'Job deleted successfully');
            } else {
                return redirect()->back()->with('error', 'Job not found');
            }
        } catch (Exception $e) {
            return redirect()->back()->with('error', 'Error: ' . $e->getMessage());
        }
    }
}
